<?php

namespace App\Http\Controllers;

use App\Models\BookableProduct;
use App\Models\TempBookingItem;
use App\Models\BookingTime;
use App\Models\PaymentItem;
use App\Models\Product;
use App\Services\BookingService;
use Carbon\Carbon;
use DateTime;
use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;

class ApiController extends Controller
{
    private $bookingService;

    function __construct()
    {
        $this->bookingService = new BookingService();
    }

    public function registerChart(Request $request) {
        $data = $request->json()->all();
        // We go through here for old menthod
        if(isset($data['date']['detail']))
        {
            $result = $this->oldMethod($data['date']['detail'], $data['product']);
        }
        // New calendar booking (used for rooms)
        else if(isset($data['calendar'])) {
            $result = $this->newMethod($data);
        }
        
        return $result;
    }

    public function updateChart(Request $request) {
        $data = $request->json()->all();
        $result = '';
        $bookingItems = [];
        if($this->validateUpdateRequest($data)) {
            $result = $data['uid'];
            foreach($data['products'] as $product) {
                $bookingInfo = TempBookingItem::where('uid', '=', $result)->where('product_id', '=', $product['product_id'])->first()->toArray();
                $productInfo = Product::where('id', $product['product_id'])->first()->toArray();
                $start = $bookingInfo['startTime'];
                $end = $bookingInfo['endTime'];
                $nights = $this->getNightsCount($bookingInfo['startTime'], $bookingInfo['endTime']);
                $options = $product['extra_options'];
                $extraOptions = isset($options[0]['ext_productGroup_id']) ? $this->GetExtraOptionsForProductGroup($options[0]['ext_productGroup_id']) : [];
                foreach($options as $option) {
                    if($option['selected']) {
                        $selectedOption = null;
                        foreach($extraOptions as $o) {
                            if($o->id == $option['id']) {
                                $selectedOption = json_decode(json_encode($o), true);
                            }
                        }

                        $toCreateBookingItemTimes = [];
                        // Since repeat every day is set and it most often
                        if(boolval($selectedOption['repeat_every_day']) || boolval($selectedOption['repeat_morning']) || boolval($selectedOption['repeat_evening'])) {
                            // Need to look if extra product is bookable early or late in day
                            // Need to loop over every date and based on if option is early or late in day and that is within booking boundries
                            $startDate = (new DateTime($start))->modify('midnight');
                            $endDate = (new DateTime($end))->modify('midnight');

                            if(boolval($selectedOption['repeat_morning'])) {
                                $s = new DateTime($start);
                                $sL = (new DateTime($start))->setTime(12,0);

                                // We want to check if start time is greater than lunch on start day, 
                                //  if so we do not want to create morning booking item on first day.
                                if($s > $sL) {
                                    $startDate->modify('+1 day');
                                }
                            }
                            if(boolval($selectedOption['repeat_evening'])) {
                                $e = new DateTime($endDate);
                                $eL = (new DateTime($endDate))->setTime(12,0);

                                // We want to check if end time is less than lunch on end day, 
                                //  if so we do not want to create evening booking item on last day.
                                if($e < $eL) {
                                    $startDate->modify('-1 day');
                                }
                            }
                            $days = iterator_to_array(new \DatePeriod($startDate, new \DateInterval('P1D'), $endDate->modify('+1 day')));
                            foreach($days as $day) {
                                $i = [
                                    'start' => $day->format('Y-m-d') . ' ' .$selectedOption['start'],
                                    'end' => $day->format('Y-m-d') . ' ' .$selectedOption['end'],
                                    'text' => $day->format('Y-m-d')
                                ];
                                $toCreateBookingItemTimes[] = $i;
                            }


                        }
                        else {
                            $toCreateBookingItemTimes = [['start' => $start, 'end' => $end]];
                        }

                        foreach($toCreateBookingItemTimes as $i) {
                            $optionProduct = $this->getDefaultPaymentForForProduct($selectedOption['product_id']);
                            $days = $this->getDayCount($optionProduct['unit'], $nights);
                            $price = $optionProduct['price'];

                            $text = isset($i['text']) ? $optionProduct['productName'] . ' - ' . $i['text'] : $optionProduct['productName'] . ' - '. $bookingInfo['externComments'];
                            // Need to create booking items for the options
                            $bookingItem = [
                                'product_id' => $selectedOption['product_id'],
                                'productGroup_id' => 0,
                                'quantity' => $option['value'],
                                'days' => $days,
                                'sum' => $price * $option['value'] * $days,
                                'tax' => $optionProduct['tax'],
                                'price' => $price,
                                'startTime' => $i['start'],
                                'endTime' => $i['end'],
                                'externComments' => $text,
                                'project' => $bookingInfo['project'], 
                                'uid' => $result,
                                'payment_provider_id' => '',
                                'created_at' => Carbon::now(),
                                'updated_at' => Carbon::now()
                            ];
                            $bookingItems[] = $bookingItem;
                        }
                    }
                }
            }
            $insterted = TempBookingItem::insert($bookingItems);
            if($insterted <= 0) {
                $result = '';
            }
        }
        return $result;
    }
    
    public function updateChartAdditionalProducts(Request $request) {
        dd($request);
        $data = $request->json()->all();
        $result = '';
        $bookingItems = [];
        if($this->validateUpdateRequest($data)) {
            $result = $data['uid'];
            foreach($data['products'] as $product) {
                $productDate = DB::table('ext_pg_date AS pd')
                ->select('pd.id', 'pd.date', 'pd.start_time', 'pd.end_time')
                ->where('pd.id', '=', $product['date_id'])
                ->whereNull('pd.deleted_at')
                ->first();

                $bookingInfo = TempBookingItem::where('uid', '=', $result)->first()->toArray();

                $date = new DateTime($productDate->date);
                $startTime = new DateTime($productDate->start_time);
                $endTime = new DateTime($productDate->end_time);
                $start = new DateTime($date->format('Y-m-d') .' ' .$startTime->format('H:i:s'));
                $end = new DateTime($date->format('Y-m-d') .' ' .$endTime->format('H:i:s'));
                $options = $product['payment_options']['options'];
                
                foreach($options as $option) {
                    if($option['value'] > 0) {
                        $paymentInfo = DB::table('ext_pg_paymentItem AS pi')
                        ->select('pi.tax', 'pi.price', 'p.unit', 'p.id AS product_id', 'pi.title AS piTitle', 'p.productName', 'bp.title')
                        ->join('ext_pg_paymentObject AS po', 'po.id', '=', 'pi.payment_object_id')
                        ->join('ext_productGroup AS pg', 'pg.id', '=', 'po.product_group_id')
                        ->join('ext_pg_product AS pgp', 'pgp.ext_productGroup_id', '=', 'po.product_group_id')
                        ->join('ext_bookableProduct AS bp', 'bp.id', '=', 'pg.ext_bookableProduct_id')
                        ->join('product AS p', 'p.id', '=', 'pgp.product_id')
                        ->where('pi.id', '=', $option['id'])
                        ->where('pg.ext_bookableProduct_id', '=', $product['id'])
                        ->whereNull('pi.deleted_at')
                        ->first();
                        $price = $paymentInfo->price;
                        $bookingItem = [
                            'product_id' => $paymentInfo->product_id,
                            'productGroup_id' => 0,
                            'quantity' => $option['value'],
                            'days' => 1,
                            'sum' => $price * $option['value'],
                            'tax' => $paymentInfo->tax,
                            'price' => $price,
                            'startTime' => $start->format('Y-m-d H:i:s'),
                            'endTime' => $end->format('Y-m-d H:i:s'),
                            'externComments' => $paymentInfo->piTitle . ' - ' . $paymentInfo->title . ' ' . $start->format('Y-m-d') . ' ' . $start->format('H:i') . ' - ' . $end->format('H:i'),
                            'project' => $bookingInfo['project'], 
                            'uid' => $result,
                            'payment_provider_id' => '',
                            'created_at' => Carbon::now(),
                            'updated_at' => Carbon::now()
                        ];
                        $bookingItems[] = $bookingItem;
                    }
                }
            }
            $insterted = TempBookingItem::insert($bookingItems);
            if($insterted <= 0) {
                $result = '';
            }
        }
        return $result;
    }

    public function getProductsForDates($bookableProductId = 0, $start = '', $end = '') {
        $result = $this->bookingService->getProductsForDates($bookableProductId, $start, $end);
        return json_encode($result);
      }

    private function validateUpdateRequest($data) {
        $result = true;
        // Checking uid and check products (since if some one is guessing registered data they need to know what has been registered)
        $productIds = [];
        
        if(isset($data['products'])) {
            foreach($data['products'] as $product) {
                $productIds[] = $product['product_id'];
            }
        }

        if(isset($data['booked'])) {
            foreach($data['booked'] as $booked) {
                $productIds[] = $booked['product_id'];
            }
        }

        // We get all booking items connected to this uid
        $tmp = TempBookingItem::where('uid', '=', $data['uid'])->get()->toArray();
        foreach($tmp as $item) {
            // we check if every booking item is connected to same products.
            if(!in_array($item['product_id'], $productIds)) {
                $result = false;
            }
        }

        // If user has modified the request and is not sending in uid for validation
        if(!isset($data['uid']) || (isset($data['uid']) && $data['uid'] == '')) {
            $result = false;
        }

        return $result;
    }

    private function getNightsCount($start, $end) {
        $s_date = new DateTime($start);
        $e_date = new DateTime($end);
        $interval = $s_date->diff($e_date);
        $nights = $interval->days;
        if($interval->h > 12) {
            $nights++;
        }
        return $nights;
    }

    private function newMethod($data) {
        $result = '';
        $uid = '';
        $main_items = [];
        $productGroups = isset($data['calendar']) ? $data['calendar'] : [];
        $uid = $this->guidv4();
        $bookingItems = [];
        foreach($productGroups as $productGroup) {
            $current_count = 0;
            if(isset($productGroup['payment_options']['options'])) {
                foreach($productGroup['payment_options']['options'] as $item) {
                    if($item['value'] > 0) {
                        $main_items[] = [
                            'productGroupId' => $productGroup['id'], 
                            'optionId' => $item['id'], 
                            'count' => $item['value'],
                            'timeInfo' => $productGroup['timeInfo'],
                            'eventUid' => isset($productGroup['eventUid']) ? $productGroup['eventUid'] : null
                        ];
                        $current_count = $item['value'];
                    }
                }
                if($current_count > 0 && isset($productGroup['extra_options'])) {
                    foreach($productGroup['extra_options'] as $e) {
                        if(isset($e['selected']) && $e['selected'] == 1) {
                            $main_items[] = [
                                'extraId' => $e['id'], 
                                'count' => $current_count,
                                'timeInfo' => $productGroup['timeInfo']
                            ];
                        }
                    }
                }
            }
        }

        // Need to set the default clock on start and end time.
        $defaultTimeValues = DB::table('listOfValues')
            ->select('type', 'value')
            ->where('type', '=', 'Starttid')
            ->orWhere('type', '=', 'Sluttid')
            ->get()
            ->toArray();

        $startTime = '00:00';
        $endTime = '00:00';
        foreach($defaultTimeValues as $val) {
            if($val->type == 'Starttid') {
                $startTime = $val->value;
            }
            if($val->type == 'Sluttid') {
                $endTime = $val->value;
            }
        }

        $data['start'] .= " ". $startTime . ":00";        
        $data['end'] .= " ". $endTime . ":00";

        foreach($main_items as $item) {
            $i = $this->bookingService->getProductToBook($item, $data['start'], $data['end'], $uid);
            foreach($i as $it) {
                $bookingItems[] = $it;
            }
        }
        $insterted = TempBookingItem::insert($bookingItems);
        $result = '';
        if($insterted > 0) {
            $result = $uid;
        }

        return $result;
    }

    private function oldMethod($data, $product) {
        $main_items = [];
        $detail = isset($data) ? $data : [];
            if(isset($detail['payment_options']['options'])) {
                foreach($detail['payment_options']['options'] as $t) {
                    if($t['value'] > 0) {
                        $item = ['bookableProduct_Id' => $product['id'], 'optionId' => $t['id'], 'count' => $t['value'], 'time_id' => $detail['time_id']];
                        $main_items[] = $item;
                    }
                }
    
                if(isset($detail['additional_products'])) {
                    foreach($detail['additional_products'] as $e) {
                        if(isset($e['selected']) && $e['selected'] == 1) {
                            if(isset($e['payment_options']['options'])) {
                                foreach($e['payment_options']['options'] as $i) {
                                    if(isset($i['value']) && $i['value'] > 0) {
                                        $item = ['bookableProduct_Id' => $e['id'], 'optionId' => $i['id'], 'count' => $i['value'], 'time_id' => $e['time_id']];
                                        $main_items[] = $item;
                                    }
                                }
                            }
                        }
                    }
                }
            }
    
            $time = [];
            if(isset($detail['time']['id'])) {
                $time = $this->itemToArray(BookingTime::where('id', $detail['time']['id'])->first());
            }
            $bookingItems = [];
            $products = [];
            $uid = $this->guidv4();
            foreach($main_items as $item) {
                $product = $this->itemToArray(BookableProduct::where('id', $item['bookableProduct_Id'])->first());
                $products[] = $product;
                $payment = $this->itemToArray(PaymentItem::where('id', $item['optionId'])->first());
                if($time['bookableProduct_id'] !== $item['bookableProduct_Id']) {
                    $t = BookingTime::where('id', $item['time_id'])->first()->toArray();
                }
                else {
                    $t = $time;
                }
                
                $bookingItem = [
                    'product_id' => isset($product['product_id']) ? $product['product_id'] : 0,
                    'bookableProduct_id' => isset($product['id']) ? $product['id'] : 0,
                    'bookingTimes_id' => $t['id'],
                    'quantity' => isset($item['count']) ? $item['count'] : 0,
                    'days' => 1,
                    'sum' => isset($item['count']) && isset($payment['price']) ? $item['count'] * $payment['price'] : 0,
                    'tax' => isset($payment['tax']) ? $payment['tax'] : 0,
                    'price' => isset($payment['price']) ? $payment['price'] : 0,
                    'startTime' => isset($t['start']) ? $t['start'] : 0,
                    'endTime' => isset($t['end']) ? $t['end'] : 0,
                    'externComments' => $product['title'] . ', ' . $t['title'] . '. ' . $payment['title'],
                    'project' => $product['title'] . ', ' . $t['title'],
                    'uid' => $uid,
                    'payment_provider_id' => '',
                    'created_at' => Carbon::now(),
                    'updated_at' => Carbon::now()
                ];
                $bookingItems[] = $bookingItem;
            }
            $insterted = TempBookingItem::insert($bookingItems);
            $result = '';
            if($insterted > 0) {
                $result = $uid;
            }
        return $result;
    }
    
    private function GetExtraOptionsForProductGroup(int $productGroupId) : array {
        $result = DB::table('ext_pg_extraProduct AS ep')
            ->select('ep.id', 'ep.product_id', 'p.productName', 'ep.max', 'ep.value', 'pli.price', 'p.unit', 'ep.checked_by_default', 'ep.repeat_every_day', 'ep.repeat_morning', 'ep.repeat_evening', 'ep.start', 'ep.end')
            ->join('product AS p', 'p.id', '=', 'ep.product_id')
            ->join('pricelistItem AS pli', 'pli.productId', '=', 'ep.product_id')
            ->join('pricelist AS pl', 'pl.id', '=', 'pli.priceListId')
            ->where('ep.ext_productGroup_id', '=', $productGroupId)
            ->where('pl.defaultPriceList', '=', 1)
            ->orderBy('ep.order')
            ->get()
            ->toArray();

        return $result;
    }
}