<?php

namespace App\Http\Controllers;

use App\Models\BookableProduct;
use App\Models\BookingSubTime;
use App\Models\BookingTime;
use App\Models\Page;
use App\Models\PaymentItem;
use App\Models\PaymentObject;
use App\Models\ProductGroup;
use Carbon\Carbon;
use DateTime;
use Illuminate\Support\Facades\DB;
use App\Services\BookingService;
use App\Services\UtilService;

class BookingController extends Controller
{
  public $bookingService;

  function __construct()
  {
    $this->bookingService = new BookingService();
    $this->utilService = new UtilService();
  }
  
  // Default page (root when solution is visited)
  public function home()
  {
    $bookableProducts = $this->bookingService->getProducts();

    // Base page info
    $data = [
      'page' => Page::where('id', 1)->first()->toArray(),
      'products' => $bookableProducts
    ];

    // Loop over products and build links and url
    foreach($data['products'] as &$product) {
      $product['url'] = url('view', [$product['id']]);
      $product['img'] = $product['image'];
    }

    return view('home', compact('data'));
  }

  // When user has clicked on a product on main page
  public function view($id)
  {
    $data = $this->bookingService->viewProduct($id);
//dd($data);
    // if(isset($data) && isset($data['calendar']) && isset($data['calendar']['enabledDates'])  && is_countable($data['calendar']['enabledDates']) && count($data['calendar']['enabledDates']) == 1){
    //     return redirect('/order/'.$data['calendar']['enabledDates'][0]);
    // }
    

    // Old logic that I need to rewrite if there is only 1 available date
    // // if(isset($times) && is_countable($times) && count($times) == 1) {
    // //   return redirect('/order/'.$times[0]['id']);
    // // }

    return view('product_view', compact('data'));
  }

  public function order($id)
  {
    $time = BookingTime::where('id', $id)->first()->toArray();
    $max = $this->GetMaxBookable($time['bookableProduct_id'], $time['id']);
    $bookableProduct = BookableProduct::where('id', $time['bookableProduct_id'])->first()->toArray();
    $paymentOptions = PaymentObject::where('id', 1)->first()->toArray();
    $paymentOptions['options'] = PaymentItem::where('paymentObject_id', $paymentOptions['id'])->get()->toArray();
    $paymentOptions['available_count'] = $max;
    $paymentOptions['fullText'] = __('fullybooked');
    foreach($paymentOptions['options'] as &$option) {
      $option['min'] = 0;
      $option['max'] = $max;
      if($option['value'] > $max) {
        $option['value'] = $max;
      }
    }
    $extraProducts = [];
    // Need to fetch linked times and then fetch products from that
    $subTimes = BookingSubTime::where('parent_bookingTimes_id', $time['id'])->get();
    foreach($subTimes as $subTime) {
      // Going over each sub time item and add as nested items
      $t = BookingTime::where('id', $subTime['child_bookingTimes_id'])->first()->toArray();
      // Getting bookable product for time
      $pr = BookableProduct::where('id', $t['bookableProduct_id'])->first()->toArray();
      // Get max bookable for this product (calculated based on what is already booked)
      $max = $this->GetMaxBookable($pr['id'], $t['id']);
      // Getting the payment object
      $po = PaymentObject::where('id', 1)->first()->toArray();
      // assaigning payment items to payment object      
      $po['options'] = PaymentItem::where('paymentObject_id', $po['id'])->get()->toArray();
      // setting min and max values for each item
      foreach($po['options'] as &$option) {
        $option['min'] = 0;
        $option['max'] = $max;
        if($option['value'] > $max) {
          $option['value'] = $max;
        }
      }      
      $po['available_count'] = $max;
      $po['fullText'] = __('fullybooked');
      $pr['sub_title'] = $t['title'] . ' ' . $t['sub_title'];
      $pr['payment_options'] = $po;
      $pr['time_id'] = $t['id'];
      $pr['item_id'] = $pr['id'].'-'.$t['id'];
      $extraProducts[] = $pr;
    }

    $data = [
      'page' => Page::where('id', 1)->first()->toArray(),
      'product' => $bookableProduct,
      'detail' => [
        'time_id' => $time['id'],
        'title' => $bookableProduct['title'],
        'time_info' => $time['title']. ': '.$time['sub_title'],
        'description' => $bookableProduct['short_description'],
        'price' => 0,
        'total_price' => 0,
        'url' => config('app.url'),
        'payment_options' => $paymentOptions,
        'additional_products' => $extraProducts,
        'translations' => [
          'modify' => __('modify')
        ],
        'time' => $time
      ]
    ];

    return view('order-product', compact('data'));
  }

  public function getExtraProducts ($id = null) {
    $extraOptionsCount = 0;
    $products = DB::table('ext_tempBookingItem AS bi')
                      ->select('bi.product_id', 'pg.title', 'bi.project', 'p.invoiceDescription', 'bi.price', 'bi.sum', 'bi.productGroup_id', 'bi.startTime', 'bi.endTime', 'p.unit')
                      ->join('product AS p', 'p.id', '=', 'bi.product_id')
                      ->join('ext_productGroup AS pg', 'bi.productGroup_id', '=', 'pg.id')
                      ->where('bi.uid', '=', $id)
                      ->whereNull('bi.deleted_at')
                      ->whereNull('pg.deleted_at')
                      ->get()
                      ->toArray();

    foreach($products as &$product) {
      // Need to get extra products for this bookingitem
      $product = json_decode(json_encode($product), true);
      $product['extra_options'] = DB::table('ext_pg_extraProduct AS ep')
            ->select('ep.id', 'ep.product_id', 'p.productName', 'ep.max', 'ep.value', 'pli.price', 'p.unit', 'ep.checked_by_default', 'ep.repeat_every_day', 'ep.repeat_morning', 'ep.repeat_evening', 'ep.ext_productGroup_id')
            ->join('product AS p', 'p.id', '=', 'ep.product_id')
            ->join('pricelistItem AS pli', 'pli.productId', '=', 'ep.product_id')
            ->join('pricelist AS pl', 'pl.id', '=', 'pli.priceListId')
            ->where('ep.ext_productGroup_id', '=', $product['productGroup_id'])
            ->where('pl.defaultPriceList', '=', 1)
            ->orderBy('ep.order')
            ->get()
            ->toArray();

      if(isset($product['unit']) && $product['unit'] == 'Dygn') {
        $product['unitText'] = "sek/natt";
      }
      else {
        $product['unitText'] = "sek";
      }

      $start = $product['startTime'];
      $end = $product['endTime'];
      $s_date = new DateTime($start);
      $e_date = new DateTime($end);
      $interval = $s_date->diff($e_date);
      $nights = $interval->days;
      if($nights == 0 && $interval->h > 0) {
        $nights = 1;
      }
      else if($interval->h > 12) {
        $nights++;
      }

      foreach($product['extra_options'] as &$extra) {
        $extra = json_decode(json_encode($extra), true);
        $extra['min'] = 0;
        $extra['selected'] = boolval($extra['checked_by_default']);
        $extra['itemSum'] = 0;
        if($extra['unit'] == 'Styck' && !(boolval($extra['repeat_every_day']) || boolval($extra['repeat_evening'])  || boolval($extra['repeat_morning']))) {
          $extra['multiplier'] = 1;
          $unit = "sek";
        }
        else if($extra['unit'] == 'Styck' && (boolval($extra['repeat_every_day']) || boolval($extra['repeat_evening']) || boolval($extra['repeat_morning']))) {
          $extra['multiplier'] = $nights;
          $unit = "sek/dag";
        }
        else if($extra['unit'] == 'Dygn') {
          $extra['multiplier'] = $nights;
          $unit = "sek/natt";
        }
        $extra['priceString'] = ' ('. intval($extra['price']) . ' ' . $unit. ')';
        $extraOptionsCount++;
      }
    }

    if($extraOptionsCount == 0) {
      return redirect('/calendar/additional/'.$id);
    }

    $data = [
      'page' => Page::where('id', 1)->first()->toArray(),
      'product_page' => ['title' => 'Lägg till extra produkter'],
      'products' => json_encode($products, true),
      'uid' => $id
  ];
    return view('calendar-extra-products', compact('data'));
  }

  public function getAdditionalProducts($id = null) {
    $additionalProducts = [];
    $bookingItems = DB::table('ext_tempBookingItem AS bi')
                          ->select('bi.startTime', 'bi.endTime', 'bi.id', 'bi.project', 'bi.sum', 'bi.quantity', 'bi.days', 'bi.product_id', 'bi.productGroup_id', 'p.productName', 'bi.externComments')
                          ->join('product AS p', 'p.id', '=', 'bi.product_id')
                          ->where('bi.uid', '=', $id)
                          ->orderBy('bi.id')
                          ->get()
                          ->toArray();
    $start = '';
    $end = '';
    // We pull the dates from first created booking item
    if(isset($bookingItems[0])) {
      $bi = $this->itemToArray($bookingItems[0]);
      $start = $bi['startTime'];
      $end = $bi['endTime'];
      $s_date = new DateTime($start);
      $e_date = new DateTime($end);
      $interval = $s_date->diff($e_date);
      $nights = $interval->days;
      $nightsText = $nights == 1 ? 'natt' : 'nätter';
      $timeInfo = $this->utilService->getDateString($s_date) . ' - ' . $this->utilService->getDateString($e_date); 
    }
    $productGroupIds = [];
    foreach($bookingItems as $item){
      if($item->productGroup_id > 0) {
        $productGroupIds[] = $item->productGroup_id;
      }
    }

    $productGroupIds = array_unique($productGroupIds);

    $additional = DB::table('ext_bookableProduct AS bp')
            ->select('bp.id', 'pg.only_specific_dates', 'pgp.product_id', 'bp.title', 'p.unit', 'pg.id AS product_group_id', 'p.productName', 'pg.description')
            ->join('ext_productGroup AS pg', 'pg.ext_bookableProduct_id', '=', 'bp.id')
            ->join('ext_pg_product AS pgp', 'pgp.ext_productGroup_id', '=', 'pg.id')
            ->join('product AS p', 'p.id', '=', 'pgp.product_id')
            ->whereNotIn('pg.id', $productGroupIds)
            ->where('p.status', '=', 'Aktiv')
            ->whereNull('pg.deleted_at')
            ->get()
            ->toArray();

    foreach($additional as $p) {
      $allowDates = [];
      $excludeDates = [];
      $add = false;

      if($p->only_specific_dates == 1) {
        $allowDates = DB::table('ext_pg_date AS pd')
                            ->selectRaw('pd.id, pd.date, TIME_FORMAT(pd.start_time, "%H:%i") AS start_time, TIME_FORMAT(pd.end_time, "%H:%i") AS end_time')
                            ->where('pd.product_group_id', '=', $p->product_group_id)
                            ->whereBetween('pd.date', array($start,$end))
                            ->whereNull('deleted_at')
                            ->get()
                            ->toArray();
        if(isset($allowDates) && count($allowDates) > 0) {
          foreach($allowDates as &$d)
          {
            $date = new DateTime($d->date);
            $d->text = $this->utilService->getDateString($date);
          }
          //$p->allowed_dates = $allowDates;
          $add = true;
        }
      }
      else {
        $excludeDates = DB::table('ext_pg_date AS pd')
                            ->select('pd.id', 'pd.date')
                            ->where('pd.product_group_id', '=', $p->product_group_id)
                            ->get()
                            ->toArray();
        if(isset($excludeDates) && count($excludeDates) > 0) {
          $p->excluded_dates = $excludeDates;
        }
        $add = true;
      }

      if($add) {
        $productGroup = $this->bookingService->getProductGroupWithPaymentInfo([], $p->product_group_id, $start, $end, $timeInfo, $nights);
        if(isset($productGroup)) {
            foreach($allowDates as $date) {
            $i = $this->itemToArray($p);
            $i['price'] = $productGroup['price'];
            $i['timeInfo'] = $productGroup['timeInfo'];
            $i['nights'] = $productGroup['nights'];
            $i['payment_options'] = $productGroup['payment_options'];
            $i['date'] = $date->text;
            $i['date_id'] = $date->id;
            $i['time'] = $date->start_time . ' - ' . $date->end_time;
            $additionalProducts[] = $i;
            }
        }
      }
    }

    if(!isset($additionalProducts) || count($additionalProducts) == 0) {
      return redirect('/checkout/create/'.$id);
    }

    $data = ['page' => Page::where('id', 1)->first()->toArray(),
              'product_page' => ['title' => 'Redan bokad', 'title2' => 'Other products that might interest you'],
              'alreadyBooked' => json_encode($bookingItems, true),
              'additionalProducts' => json_encode($additionalProducts, true),
              'uid' => $id];
    // Get all other products that are bookable for dates.
    // select form products that are bookable for selected dates and product id is not in list of temp_bookingItem_id for uid.
    return view('calendar-additional-products', compact('data'));
  }


  

  // private function getProductGroupWithPaymentInfo($productGroup = [], $productGroupId = null, $start, $end, $timeInfo, $nights) {
  //   if(isset($productGroupId)) {
  //     $productGroup = ProductGroup::where('id', '=', $productGroupId)->first()->toArray();
  //   }

  //   $product = $this->getSingleFreeProductFromGroup($productGroup['id'], $start, $end, []);
  //   $item = null;
  //   if(isset($product) && count($product) > 0) {
  //     $payment = $this->getDefaultPaymentForForProduct($product['id']);
  //     $paymentObject = PaymentObject::where('product_group_id', $productGroup['id'])->first();
  //     $paymentItems = $paymentObject->items()->get();

  //     $payment_options = $paymentObject->toArray();
  //     $payment_options['options'] = $paymentItems->toArray();


  //     if(isset($payment['unit']) && $payment['unit'] == 'Dygn') {
  //       $unit = "sek/natt";
  //     }
  //     else if(isset($payment['unit']) && $payment['unit'] == 'Styck') {
  //       $unit = "sek";
  //     }
      
  //     $payment_options['full_text'] = __('fullybooked');
  //     $payment_options['available_count'] = $this->getProductCountForGroup($productGroup['id'], $start, $end);

  //     foreach($payment_options['options'] as &$option) {
  //       if($option['title'] == '') {
  //         $option['title'] = $option['price']." ".$unit;
  //       }
  //       else {
  //         $option['title'] .= " (".$option['price']." ".$unit.")";
  //       }
  //       $option['min'] = 0;
  //       $option['value'] = 0;
  //       $option['max'] = $payment_options['available_count'];
  //     }

  //     $item = $productGroup;
  //     $item['price'] = 0;
  //     $item['timeInfo'] = $timeInfo;
  //     $item['nights'] = $nights;
  //     $item['payment_options'] = $payment_options;
  //   }

  //   return $item;
  // }
}
