<?php

namespace App\Http\Controllers;

use App\Models\BookableProduct;
use App\Models\Booking;
use App\Models\BookingItem;
use App\Models\BookingTime;
use App\Models\TempBookingItem;
use Carbon\Carbon;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;
use Illuminate\Support\Facades\DB;

class Controller extends BaseController
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests;

    // public function GetMaxBookable($bookingItem): int {

    //   $bookableProduct = BookableProduct::where('id', $bookableProductId)->first();
    //   $bookingTime = BookingTime::where('id', $timeId)->first();
    //   $productIds = ''; // select product_id from ext_bookableProduct_products where ext_bookableProduct_id = $bookableProductId
    //   // Need to create special logic for when there are multiple products connected to the bookable product
    //   $bookingItems = BookingItem::where('productId', '=', $bookingItem['product_id'])->where('startTime', '=', $bookingTime->start)->where('endTime', '=', $bookingTime->end)->get();
  
    //   $result = $bookableProduct->max;
    //   foreach($bookingItems as $bookingItem) {
    //       $booking = Booking::where('id', $bookingItem->bookingId)->first();
    //       if($booking->status != 'Makulerad') {
    //           $result -= $bookingItem->quantity;
    //       }            
    //   }
  
    //   $newer = Carbon::now()->subMinute(15)->format('Y-m-d H:i:s');
    //   if($bookingItem['payment_provider'] == '') {
    //       $tempBookingItems = TempBookingItem::whereNull('deleted_at')->where('product_id', '=', $bookableProduct->product_id)->where('startTime', '=', $bookingTime->start)->where('endTime', '=', $bookingTime->end)->where('created_at', '>=', $newer)->get();
    //   }
    //   else {
    //       $tempBookingItems = TempBookingItem::whereNull('deleted_at')->where('payment_provider_id', '<>', $bookingItem['payment_provider_id'])->where('product_id', '=', $bookableProduct->product_id)->where('startTime', '=', $bookingTime->start)->where('endTime', '=', $bookingTime->end)->where('created_at', '>=', $newer)->get();
    //   }
    //   foreach($tempBookingItems as $bookingItem) {
    //     $result -= $bookingItem->quantity;
    //   }
  
    //   if($result < 0) {
    //     $result = 0;
    //   }
    //   return $result;
    // }

    public function itemToArray($item) {
      if($item == null) {
          $item = [];
      }
      else if(isset($item) && !is_object($item)) {
          $item = $item->toArray();
      }
      else {
          $item = json_decode(json_encode($item), true);
      }
      return $item;
  }

  public function guidv4($data = null) {
      // Generate 16 bytes (128 bits) of random data or use the data passed into the function.
      $data = $data ?? random_bytes(16);
      assert(strlen($data) == 16);
  
      // Set version to 0100
      $data[6] = chr(ord($data[6]) & 0x0f | 0x40);
      // Set bits 6-7 to 10
      $data[8] = chr(ord($data[8]) & 0x3f | 0x80);
  
      // Output the 36 character UUID.
      return vsprintf('%s%s-%s-%s-%s-%s%s%s', str_split(bin2hex($data), 4));
  }

  public function getProductCount($productId, $start, $end, $uid = null) : int {
    $products = $this->getFreeProducts($productId, $start, $end, $uid);
    $totalCount = 0;
    foreach($products as $product) {
      $totalCount += $product['totalQuantity'];
    }
    return $totalCount;
  }

  public function getProductCountForGroup($productGroupId, $start, $end) : int {
    $products = $this->getFreeProductsFromGroup($productGroupId, $start, $end);
    $totalCount = 0;
    foreach($products as $product) {
      $totalCount += $product['totalQuantity'];
    }
    return $totalCount;
  }

  public function getFreeProducts($productId, $start, $end, $uid) : array {
    /**
       * !! NOTE FOR DEBUGGING !!
       *  For viewing of sql query change ->get() over to ->toSql() that way you get output of query that is running.
       */
      // Only look at temp items newer than 15 min
      $newer = Carbon::now()->subMinute(15)->format('Y-m-d H:i:s');

      // Look up all booked products in group for given time periode
      $booked = DB::table('product AS p')
          ->select('p.id')
          ->leftJoin('bookingItem AS i', 'i.productId', '=', 'p.id')
          ->where('p.id', '=', $productId)
          ->where('p.hidden_product', '=', 0)
          ->where(function($query) use (&$start, &$end) {
              $query->whereBetween('i.startTime', [$start, $end])
                  ->orWhereBetween('i.endTime', [$start, $end])
                  ->orWhere(function($subq1) use (&$start, &$end) {
                      $subq1->where('i.startTime', '<=', $start)
                          ->where('i.endTime', '>=', $end);
                  })
                  ->orWhere(function($subq2) use (&$start, &$end) {
                      $subq2->where('i.startTime', '>=', $start)
                          ->where('i.endTime', '<=', $end);
                  });
          })
          ->whereNotIn('i.bookingId', function($notInQuery) {
              $notInQuery->select('b.id')
                  ->from('booking AS b')
                  ->where('b.status', '=', 'Makulerad');
          })->get();
      // Look at all temp booked products in group for given time periode
      $tempBooked = DB::table('product AS p')
              ->select('p.id')
              ->leftJoin('ext_tempBookingItem AS i', 'i.product_id', '=', 'p.id')
              ->where(function($query) use (&$start, &$end) {
                  $query->whereBetween('i.startTime', [$start, $end])
                      ->orWhereBetween('i.endTime', [$start, $end])
                      ->orWhere(function($subq1) use (&$start, &$end) {
                          $subq1->where('i.startTime', '<=', $start)
                              ->where('i.endTime', '>=', $end);
                      })
                      ->orWhere(function($subq2) use (&$start, &$end) {
                          $subq2->where('i.startTime', '>=', $start)
                              ->where('i.endTime', '<=', $end);
                      });
              })
              ->where('p.id', '=', $productId)
              ->where('p.hidden_product', '=', 0)
              ->where('i.updated_at', '>', $newer)
              ->whereNull('i.deleted_at')
              ->where('i.uid', '<>', $uid)
              ->get();

      // Select all products in group that do not have bookings or temp bookings from queries above
      $products = DB::table('product AS p')
                  ->select('p.*')
                  ->where('p.id', '=', $productId)
                  ->where('p.hidden_product', '=', 0)
                  ->whereNotIn('p.id', $this->itemToArray($booked))
                  ->whereNotIn('p.id', $this->itemToArray($tempBooked))
                  ->orderBy('disPos')
                  ->get();
      $products = $this->itemToArray($products);
      return $products;
  }

  public function getProductsFromGroupWithCounts($productGroupId, $start, $end) : array {
    // Only look at temp items newer than 15 min
    $newer = Carbon::now()->subMinute(15)->format('Y-m-d H:i:s');

    $sqlResult = DB::select("SELECT 
                        p.id, 
                        p.totalQuantity, 
                        CAST(NVL(i.bookedQuantity,0) AS SIGNED) AS bookedQuantity, 
                        CAST(NVL(i2.tempQuantity,0) AS SIGNED) AS tempQuantity 
                      FROM product p
                      JOIN ext_pg_product pgp ON pgp.product_id = p.id
                      JOIN ext_productGroup pg ON pg.id = pgp.ext_productGroup_id
                      -- BookingItem
                      LEFT JOIN (SELECT SUM(bi.quantity) AS bookedQuantity, bi.productId FROM bookingitem bi
                          WHERE (
                            bi.startTime BETWEEN '$start' AND '$end'
                            OR bi.endTime BETWEEN '$start' AND '$end'
                            OR (bi.startTime <= '$start' AND bi.endTime >= '$end')
                            OR (bi.startTime >= '$start' AND bi.endTime <= '$end'))
                            AND bi.bookingId NOT IN (SELECT id FROM booking WHERE STATUS = 'Makulerad')
                          GROUP BY bi.productId) i ON i.productId = p.id
                      -- TempBookingItem
                      LEFT JOIN (SELECT SUM(tbi.quantity) AS tempQuantity, tbi.product_id FROM ext_tempBookingItem tbi
                          WHERE (
                            tbi.startTime BETWEEN '$start' AND '$end'
                            OR tbi.endTime BETWEEN '$start' AND '$end'
                            OR (tbi.startTime <= '$start' AND tbi.endTime >= '$end')
                            OR (tbi.startTime >= '$start' AND tbi.endTime <= '$end'))
                            AND tbi.updated_at > '$newer'
                            AND tbi.deleted_at IS NULL
                          GROUP BY tbi.product_id) i2 ON i2.product_id = p.id
                      WHERE pg.id = $productGroupId
                      AND p.hidden_product = 0
                      ORDER BY disPos");

      $result = [];
      // We want to loop over the data coming from the database
      foreach($sqlResult as $product) {
        $bookableCount = $product->totalQuantity - $product->bookedQuantity - $product->tempQuantity;
        $item = ['ProductId' => $product->id, 'BookableCount' => $bookableCount ];
        $result[] = $item;
      }

      return $result;
  }

  public function getFreeProductsFromGroup($productGroupId, $start, $end) : array {
      /**
       * !! NOTE FOR DEBUGGING !!
       *  For viewing of sql query change ->get() over to ->toSql() that way you get output of query that is running.
       */
      // Only look at temp items newer than 15 min
      $newer = Carbon::now()->subMinute(15)->format('Y-m-d H:i:s');

      // Look up all booked products in group for given time periode
      $booked = DB::table('ext_productGroup AS pg')
          ->select('p.id')
          ->join('ext_pg_product AS pgg', 'pgg.ext_productGroup_id', '=', 'pg.id')
          ->join('product AS p', 'p.id', '=', 'pgg.product_id')
          ->leftJoin('bookingItem AS i', 'i.productId', '=', 'p.id')
          ->where('pg.id', '=', $productGroupId)
          ->where('p.hidden_product', '=', 0)
          ->where(function($query) use (&$start, &$end) {
              $query->whereBetween('i.startTime', [$start, $end])
                  ->orWhereBetween('i.endTime', [$start, $end])
                  ->orWhere(function($subq1) use (&$start, &$end) {
                      $subq1->where('i.startTime', '<=', $start)
                          ->where('i.endTime', '>=', $end);
                  })
                  ->orWhere(function($subq2) use (&$start, &$end) {
                      $subq2->where('i.startTime', '>=', $start)
                          ->where('i.endTime', '<=', $end);
                  });
          })
          ->whereNotIn('i.bookingId', function($notInQuery) {
              $notInQuery->select('b.id')
                  ->from('booking AS b')
                  ->where('b.status', '=', 'Makulerad');
          })->get();
      // Look at all temp booked products in group for given time periode
      $tempBooked = DB::table('ext_productGroup AS pg')
              ->select('p.id')
              ->join('ext_pg_product AS pgg', 'pgg.ext_productGroup_id', '=', 'pg.id')
              ->join('product AS p', 'p.id', '=', 'pgg.product_id')
              ->leftJoin('ext_tempBookingItem AS i', 'i.product_id', '=', 'p.id')
              ->where(function($query) use (&$start, &$end) {
                  $query->whereBetween('i.startTime', [$start, $end])
                      ->orWhereBetween('i.endTime', [$start, $end])
                      ->orWhere(function($subq1) use (&$start, &$end) {
                          $subq1->where('i.startTime', '<=', $start)
                              ->where('i.endTime', '>=', $end);
                      })
                      ->orWhere(function($subq2) use (&$start, &$end) {
                          $subq2->where('i.startTime', '>=', $start)
                              ->where('i.endTime', '<=', $end);
                      });
              })
              ->where('pg.id', '=', $productGroupId)
              ->where('p.hidden_product', '=', 0)
              ->where('i.updated_at', '>', $newer)
              ->whereNull('i.deleted_at')
              ->get();

      // Select all products in group that do not have bookings or temp bookings from queries above
      $products = DB::table('ext_productGroup AS pg')
                  ->select('p.*')
                  ->join('ext_pg_product AS pgg', 'pgg.ext_productGroup_id', '=', 'pg.id')
                  ->join('product AS p', 'p.id', '=', 'pgg.product_id')
                  ->where('pg.id', '=', $productGroupId)
                  ->where('p.hidden_product', '=', 0)
                  ->whereNotIn('p.id', $this->itemToArray($booked))
                  ->whereNotIn('p.id', $this->itemToArray($tempBooked))
                  ->orderBy('disPos')
                  ->get();
      $products = $this->itemToArray($products);
      return $products;
  }

  public function getDefaultPaymentForForProduct($productId) {
    $payment = $this->itemToArray(DB::table('product AS p')
      ->select('p.id','pli.price','pli.tax','p.unit', 'p.productName')
      ->leftjoin('priceListItem AS pli', 'p.id', '=', 'pli.productId')
      ->leftjoin('priceList AS pl', 'pli.priceListId', '=', 'pl.id')
      ->where('p.id', '=', $productId)
      ->where('pl.defaultPriceList', '=', 1)
      ->first());
    return $payment;
  }

  public function getPaymentForForProduct($productId, $optionId) {
    $payment = $this->itemToArray(DB::table('product AS p')
      ->select('p.id', 'pi.id AS itemId', 'pi.title','p.unit', 'p.productName', 'pi.price', 'pi.tax')
      ->join('ext_pg_product AS pgp', 'pgp.product_id', '=', 'p.id')
      ->join('ext_pg_paymentObject AS po', 'po.product_group_id', '=', 'pgp.ext_productGroup_id')
      ->join('ext_pg_paymentItem AS pi', 'pi.payment_object_id', '=', 'po.id')
      ->where('p.id', '=', $productId)
      ->where('pi.id', '=', $optionId)
      ->whereNull('pi.deleted_at')
      ->first());
    return $payment;
  }

    public function GetDatePickerTranslations() {
      $result = [
        'date.start_end_same' => __('date.start_end_same'),
        'date.cannot_book_all_days' => __('date.cannot_book_all_days'),
        'date.end_before_start' => __('date.end_before_start'),
        'date.footer_full_day' => __('date.footer_full_day'),
        'date.footer_full_days' => __('date.footer_full_days'),
        'date.footer_arrival' => __('date.footer_arrival'),
        'date.footer_book' => __('date.footer_book'),
        'date.unable_to_book' => __('date.unable_to_book')
      ];
      return $result;
    }

    public function getDayCount($unit, $days) {
      $result = 0;
      if($unit == 'Dygn') {
        $result = $days == 0 ? 1 : $days;
      }
      else if($unit == 'Styck') {
        $result = 1;
      }

      return $result;
    }
}
