<?php

namespace App\Http\Controllers;

use App\Models\Address;
use App\Models\BookableProduct;
use App\Models\Booking;
use App\Models\BookingItem;
use App\Models\TempBookingItem;
use App\Models\BookingType;
use App\Models\ContactPerson;
use App\Models\Customer;
use App\Models\Page;
use App\Models\PaymentTerm;
use App\Models\Pricelist;
use App\Services\BookingService;
use Carbon\Carbon;
use Illuminate\Support\Facades\Http;

class KlarnaController extends Controller
{
  private string $authKeyBase64;
  private string $country;
  private string $currency;
  private string $locale;


  function __construct() {
    $this->bookingService = new BookingService();
    $this->authKeyBase64 = config('app.klarna-authentication');
    $this->country = 'SE';
    $this->currency = 'SEK';
    $this->locale = 'sv-SE';
  }

  public function createKlarnaOrder($id=null) {
    $bookingItems = TempBookingItem::where('uid', $id)->get()->toArray();

    $totalSum = 0;
    $totalTax = 0;
    $orderLines = [];
    foreach($bookingItems as $item) {
      $sum = intval($item['sum']*100);
      $tax = intval((intval($item['sum']*100)/(10000+intval($item['tax']*100))) * intval($item['tax']*100));
      $quantity = $item['quantity'];
      $item_price = $item['days'] > 0 ? intval($item['price']*100) * $item['days']: intval($item['price']*100);
      $orderLine = [
        'type' => 'digital',
        'reference' => $id,
        'name' => $item['externComments'],
        'quantity' => $quantity,
        'quantity_unit' => 'pcs',
        'unit_price' => $item_price,
        'tax_rate' => intval($item['tax']*100),
        'total_amount' => $sum,
        'total_discount_amount' => 0,
        'total_tax_amount' => $tax
      ];
      $totalSum += $sum;
      $totalTax += $tax;
      $orderLines[] = $orderLine;
    }

    $event_name = $bookingItems[0]['externComments'];
    $event_company = 'Nya Slottet Bjärka-Säby';
    $genre_of_event = '';
    $arena_name = 'Nya Slottet Bjärka-Säby';
    $street_address = 'Pingstkyrkan';
    $postal_code = '589 64';
    $city = 'Sturefors';
    $country = 'Sverige';
    $start_time = $bookingItems[0]['startTime'];
    $end_time = $bookingItems[0]['endTime'];
    $access_controlled_venue = false;
    $ticket_delivery_method = '';
    $ticket_delivery_recipient = '';
    $affiliate_name = '';

    // Attachment data used for fraud checking, very strange this JSON is returned as JSON string. 
    $attachmentBody = [
      'event' => [
        'event_name' => $event_name,
        'event_company' => $event_company,
        // 'genre_of_event' => $genre_of_event,
        'arena_name' => $arena_name,
        'arena_location' => [
          'street_address' => $street_address,
          'postal_code' => $postal_code,
          'city' => $city,
          'country' => $country,
        ],
        // 'start_time' => $start_time,
        // 'end_time' => $end_time,
        'access_controlled_venue' => $access_controlled_venue,
        // 'ticket_delivery_method' => $ticket_delivery_method,
        // 'ticket_delivery_recipient' => $ticket_delivery_recipient,
        // 'affiliate_name' => $affiliate_name,
      ]

    ];

    $attachmentBodyString = json_encode($attachmentBody);
    //$attachmentBodyString = str_replace('"', '\"', $attachmentBodyString);
    $attachmentBodyString = str_replace('event":{"', 'event":[{"', $attachmentBodyString);
    $attachmentBodyString = str_replace('}}', '}]}', $attachmentBodyString);

    $data = [
      'purchase_country' => $this->country,
      'purchase_currency' => $this->currency,
      'locale' => $this->locale,
      'order_amount' => $totalSum,
      'order_tax_amount' => $totalTax,
      'order_lines' => $orderLines,
      'merchant_urls' => [
        'terms' => url('terms'),
        'checkout' => url('checkout/{checkout.order.id}'),
        'confirmation' => url('checkout/confirm/{checkout.order.id}'),
        'push' => url('checkout/push/{checkout.order.id}'),
      ],
      'attachment' => [
        'body' => $attachmentBodyString,
        'content_type' => 'application/vnd.klarna.internal.emd-v2+json'
    
      ],
    ];

    //dd($data);

    $response = Http::withBasicAuth(config('app.klarna-user'),config('app.klarna-pass'))
      ->post(config('app.klarna-endpoint').'checkout/v3/orders', $data);

      if($response->status() == 401) {
        dd('Klarna authentication fail');
      }
      
    $response = json_decode($response->getBody());
    
    TempBookingItem::where('uid', $id)->update(['payment_provider_id' => $response->order_id]);
    
    $data = [
      'page' => Page::where('id', 1)->first()->toArray(),
      'translations' => [
        'klarna.warning' => __('klarna.warning')
      ],
      'snippet' => $response->html_snippet,
      'orderId' => $response->order_id
    ];
    return view('klarna-checkout', compact('data','response'));
  }
  
    public function createOrder($auth_token = null)
    {
        echo $auth_token;
    }

    public function terms()
    {
      $data = [
        'page' => Page::where('id', 1)->first()->toArray()
      ];
      return view('terms', compact('data'));
    }

    public function getKlarnaOrderResponse($id=null)
    {
      // Need to start check if there is actually order in database with this id, since if user reloded we do not want to show the page again
      $items = TempBookingItem::where('payment_provider_id', $id)->get();
      $stopRun = true;
      foreach($items as $ti) {
        $stopRun = false;
      }

      if($stopRun) {
        return redirect('/');
      }

      $pageTitle = '';
      $title = '';
      $pageDescription = '';
      $response = Http::withHeaders(['Authorization' => 'Basic '.$this->authKeyBase64])
      ->get(config('app.klarna-endpoint').'checkout/v3/orders/'.$id, []);
      $response = json_decode($response->getBody());
      $snippet = $response->html_snippet;

      $isBookable = $this->checkIfBookingIsBookable($id);
      if($isBookable) {
        //dd('Stoping request before it creates item in our booking system');
        $this->createBookingInBookingSystem($id);
        $this->confirmDelivery($id);
      }
      else {
        $this->refundBooking($id);
        TempBookingItem::where('payment_provider_id', $id)->delete();
        $error = __('klarna.fullybooked');
      }

      
      $data = [
        'page' => Page::where('id', 1)->first()->toArray(),
        'snippet' => !isset($error) ? $response->html_snippet : '',
        'orderId' => $response->order_id,
        'error' => isset($error) ? $error : null
      ];

      return view('klarna-order-confirm', compact('data'));
    }

    public function push($id=null)
    {
      // We even need this endpoint since we are getting response from Klarna directly and react right away. Should not need push to happen.
      echo $id . ' push';
    }

    private function checkIfBookingIsBookable($payment_provider_id) : bool {
      $result = false;
      $bookingItems = TempBookingItem::where('payment_provider_id', $payment_provider_id)->get()->toArray();
      foreach($bookingItems as $bookingItem) {
        $max = $this->bookingService->getProductBookableCount($bookingItem['product_id'], $bookingItem['startTime'], $bookingItem['endTime'], $bookingItem['uid']);
        if($max < $bookingItem['quantity']) {
          $result = false;
          break;
        }
        else  {
          $result = true;
        }
      }

      return true; //$result;
    }

    private function refundBooking($id) {
      $response = Http::withHeaders(['Authorization' => 'Basic '.$this->authKeyBase64])
      ->post(config('app.klarna-endpoint').'ordermanagement/v1/orders/'.$id.'/cancel');
      $response = json_decode($response->getBody());
      // Need to add process here so that we get manual interaction to refund the user.
    }

    private function confirmDelivery($id) {
      // Get information on checkout so that we know how much was payed
      $checkout = Http::withHeaders(['Authorization' => 'Basic '.$this->authKeyBase64])
      ->get(config('app.klarna-endpoint').'checkout/v3/orders/'.$id);
      $booking = json_decode($checkout->getBody());

      $order_lines = $booking->order_lines;
      $order_amount = $booking->order_amount;

      // Then we notify Klarna that we accept full amount of booking, if we do not do this Klarna treats this as we have not shipped the product to user.
      $data = ['captured_amount' => $order_amount, 'order_lines' => $order_lines];
      $response = Http::withHeaders(['Authorization' => 'Basic '.$this->authKeyBase64])
      ->post(config('app.klarna-endpoint').'ordermanagement/v1/orders/'.$id.'/captures', $data);
      $response = json_decode($response->getBody());
    }

    private function createBookingInBookingSystem($payment_provider_id) {
      // Get information about order from Klarna
      $response = Http::withHeaders(['Authorization' => 'Basic '.$this->authKeyBase64])
      ->get(config('app.klarna-endpoint').'ordermanagement/v1/orders/'.$payment_provider_id);
      $response = json_decode($response->getBody());

      $billing = isset($response->billing_address) ? $response->billing_address : null;

      // We look up email used against customers in database
      if(isset($billing->email)) {
        $customer = Customer::where('email', $billing->email)->first();
      }

      // Get booking type external booking
      $bookingType = BookingType::where('shortName', 'ExB')->first();
      if(!isset($bookingType)) {
        $bookingType = BookingType::create([
          'name' => 'Extern Bokning',
          'shortName' => 'ExB',
          'type' => 'Extern',
          'color' => '',
          'status' => 'Aktiv'
        ]);
      }

      // Get the pricelist named Klarna
      $pricelist = Pricelist::where('priceListName', '	Externt prislista')->first();
      if(!isset($pricelist)) {
        $pricelist = Pricelist::create([
          'priceListName' => '	Externt prislista',
          'description' => 'Prislista anv&auml;nd f&ouml;r externt bokningar',
          'defaultPricelist' => 0,
          'status' => 'Aktiv'
        ]);
      }

      $paymentTerm = PaymentTerm::where('description', 'Externt')->first();
      if(!isset($paymentTerm)) {
        $paymentTerm = PaymentTerm::create([
          'description' => 'Externt',
          'bookingFee' => 0,
          'prepayment' => 0,
          'standard' => 0,
          'status' => 'Aktiv'
        ]);
      }

      // if we find no customer then we create one
      if(!isset($customer)) {
        $customer = Customer::create([
          'defaultPriceListId' => $pricelist->id,
          'paymentTermId' => $paymentTerm->id,
          'name' => $billing->given_name . ' ' . $billing->family_name,
          'contactName' => $billing->given_name . ' ' . $billing->family_name,
          'street' => $billing->street_address,
          'street2' => $billing->street_address2,
          'postCode' => $billing->postal_code,
          'email' => $billing->email,
          'emailInvoice' => '',
          'customerType' => 'Privat',
          'type' => 'Extern',
          'phone' => $billing->phone,
          'mobile' => '',
          'social_security_number_org_nr' => '',
          'taxType' => 'Inkl',
          'city' => $billing->city,
          'status' => 'Aktiv',
          'notes' => ''
        ]);
      }

      if(isset($customer)) {      
        $contactPerson = ContactPerson::where('email', $billing->email)->where('customerId', $customer->id)->first();
        if(!isset($contactPerson)) {
          $contactPerson = ContactPerson::create([
            'customerId' => $customer->id,
            'name' => $billing->given_name . ' ' . $billing->family_name,
            'email' => $billing->email,
            'phone' => $billing->phone,
            'mobile' => '',
            'status' => 'Aktiv'
          ]);
        }

        $address = Address::where('customerId', $customer->id)->where('type', 'standard')->first();
        if(!isset($address)) {
          $address = Address::insert([
            'customerId' => $customer->id,
            'orgName' => '',
            'street' => $billing->street_address,
            'street2' => $billing->street_address2,
            'postCode' => $billing->postal_code,
            'city' => $billing->city,
            'type' => 'standard'
          ]);
        }
        
        $bookingItems = TempBookingItem::where('payment_provider_id', $payment_provider_id)->get()->toArray();
        $start = null;
        $end = null;
        $project = null;
        foreach($bookingItems as $item) {
          $tmp_start = Carbon::createFromFormat('Y-m-d H:i:s', $item['startTime']);
          $tmp_end = Carbon::createFromFormat('Y-m-d H:i:s', $item['endTime']);
          if(!isset($project)) {
            $project = $item['project'];
          }
          if(!isset($start) && isset($tmp_start)) {
            $start = $tmp_start;
          }
          else if(isset($start) && $tmp_start->lt($start)) {
            $start = $tmp_start;
          }

          if(!isset($end) && isset($tmp_end)) {
            $end = $tmp_end;
          }
          else if(isset($end) && $tmp_end->gt($end)) {
            $end = $tmp_end;
          }
        }

        $booking = Booking::create([
          'customerId' => $customer->id,
          'priceListId' => $pricelist->id,
          'contactPersonId' => $contactPerson->id,
          'bookingTypeId' => $bookingType->id,
          'paymentTermId' => $paymentTerm->id,
          'agreement' => '',
          'project' => $project,
          'preacher' => '',
          'startTime' => $start->format('Y-m-d H:i:s'),
          'endTime' => $end->format('Y-m-d H:i:s'),
          'bookingType' => 'extern bokning',
          'type' => 'Extern',
          'description' => 'Extern bokning',
          'guestQuantity' => 0,
          'ventilation' => '',
          'status' => 'Betald',
          'notes' => '',
          'notes2' => '',
          'ourReference' => 'Extern bokning',
          'invoiceReference' => '',
          'discountPercent' => 0,
          'publicNotes' => '',
          'offering' => '',
          'meetingOrganizer' => '',
          'music' => '',
          'canticle' => '',
          'technology' => '',
          'take' => '',
          'team' => '',
          'producer' => '',
          'endedTextId' => 0,
          'pricePerPerson' => 0,
          'publish' => 0,
          'bookingCopyId' => 0,
          'createdBy' => 'Extern',
          'createdDate' => Carbon::now(),
          'lastEditedBy' => 'Extern',
          'lastEditedDate' => Carbon::now()
        ]);

        foreach($bookingItems as $item) {
          $bookingItem = BookingItem::create([
            'bookingId' => $booking->id,
            'productId' => $item['product_id'],
            'comments' => '',
            'externComments' => $item['externComments'],
            'tax' => $item['tax'],
            'price' => $item['price'],
            'quantity' => $item['quantity'],
            'days' => $item['days'],
            'provisional' => '0',
            'responsibleKitchen' => '',
            'responsibleCatering' => '',
            'quantityType' => '',
            'startTime' => $item['startTime'],
            'endTime' => $item['endTime'],
            'discountValue' => '0.00',
            'discountType' => 'percent',
            'movable' => '1',
            'altText' => '',
            'account' => '',
            'resultUnit' => '',
            'project' => $item['project'],
            'parentId' => 0,
            'eventUid' => $item['eventUid'],
            'ext_paymentItem_id' => $item['ext_pg_paymentItem_id']
          ]);

          if(isset($bookingItem)) {
            TempBookingItem::where('id', $item['id'])->delete();
          }
        }
      }
    }
}