<?php
namespace App\Services;

use Illuminate\Support\Facades\DB;
use DateTime;

// Services
use App\Services\UtilService;

// Models used
use App\Models\{Page,BookableProduct,PaymentObject,Product,ProductDate,ProductGroup};
use Carbon\Carbon;

class BookingService
{
    private $utilService;
    function __construct()
    {
        $this->utilService = new UtilService();
    }

    public function getProducts() : array {
        $result = [];
        $bookableProducts = BookableProduct::where('published', 1)->orderBy('id', 'asc')->get()->toArray();
        foreach($bookableProducts as &$item) {
            $productGroups = ProductGroup::where('ext_bookableProduct_id', $item['id'])->get()->toArray();
            //dd($productGroups);
            if(isset($productGroups[0]['only_specific_dates']) && boolval($productGroups[0]['only_specific_dates'])) {
                // We want to check if there are any dates that are not fully booked for this product
                $productGroupIds = [];
                foreach($productGroups as $pg) {
                    $productGroupIds[] = $pg['id'];
                }

                $result[] = $item;

                $i = DB::select("SELECT * FROM ext_productGroup pg");
                // $result = DB::select("SELECT 
                //                         ebi.*,
                //                         p.totalQuantity, 
                //                         CAST(NVL(i.bookedQuantity,0) AS SIGNED) AS bookedQuantity, 
                //                         CAST(NVL(i2.tempQuantity,0) AS SIGNED) AS tempQuantity,
                //                         p.totalQuantity - CAST(NVL(i.bookedQuantity,0) AS SIGNED) - CAST(NVL(i2.tempQuantity,0) AS SIGNED) AS bookableCount
                //                     FROM ext_pg_date pgd
                //                     JOIN product p on p.id = ebi.product_id
                //                     -- BookingItem
                //                     LEFT JOIN (
                //                         SELECT SUM(bi.quantity) AS bookedQuantity, bi.productId, bi.startTime, bi.endTime
                //                             FROM bookingitem bi
                //                             WHERE bi.bookingId NOT IN (SELECT id FROM booking WHERE STATUS = 'Makulerad')
                //                         GROUP BY bi.productId, bi.startTime, bi.endTime) i ON 
                //                             i.productId = p.id
                //                             AND (
                //                                 i.startTime BETWEEN ebi.startTime AND ebi.endTime
                //                                 OR i.endTime BETWEEN ebi.startTime AND ebi.endTime
                //                                 OR (i.startTime <= ebi.startTime AND i.endTime >= ebi.endTime)
                //                                 OR (i.startTime >= ebi.startTime AND i.endTime <= ebi.endTime))
                //                     -- TempBookingItem
                //                     LEFT JOIN (SELECT SUM(tbi.quantity) AS tempQuantity, tbi.product_id, tbi.startTime, tbi.endTime, tbi.id
                //                         FROM ext_tempBookingItem tbi
                //                         WHERE 
                //                             (tbi.waiting_for_payment_confirmation IS NOT NULL OR tbi.updated_at > DATE_SUB(NOW(),INTERVAL 15 MINUTE))
                //                             AND 
                //                             tbi.deleted_at IS NULL
                //                         GROUP BY tbi.product_id, tbi.startTime, tbi.endTime, tbi.id) i2 ON 
                //                             i2.product_id = p.id 
                //                             AND (
                //                                 i2.startTime BETWEEN ebi.startTime AND ebi.endTime
                //                                 OR i2.endTime BETWEEN ebi.startTime AND ebi.endTime
                //                                 OR (i2.startTime <= ebi.startTime AND i2.endTime >= ebi.endTime)
                //                                 OR (i2.startTime >= ebi.startTime AND i2.endTime <= ebi.endTime))
                //                             AND i2.id <> ebi.id
                //                     WHERE ebi.payment_provider_id = '$payment_provider_id'
                //                     AND ebi.deleted_at IS NULL");
        //return $result;

            }
            else {
                $result[] = $item;
            }
        }

        return $result;
    }

    public function getProductPage() {
        $result = [
            'titleBar' => 'Nya Slottet bokning', 
            'pageTitle' => 'Nya Slottet bokningsystem',
            'pageDescription' => 'På Nya Slottet Bjäka-Säby erbjuder vi konserter, övernattning inkl frukost och guidning.',
            'pageLogo' => 'http://pingstlinkoping.se/wp-content/uploads/2016/03/Nya-Slottet%E2%80%93Svart_smal.png',
            'products' => [
                BookableProduct::All()
            ]];
        return $result;
    }

    public function viewProduct($bookableProductId) {
        // Since main page does not list the products iteslf but bookable products that then are connected to product group that then can have many products under,
        //  we start by loading bookable product by id.
        $bookableProduct = BookableProduct::where('id', $bookableProductId)->whereNull('deleted_at')->first()->toArray();

        // Setting the correct image link
        $bookableProduct['img'] = asset($bookableProduct['image']);

        // We select from productDates table and look at product group to see if dates are only display those that are in database or exclude the dates in database
        $dates = DB::table('ext_pg_date AS pd')
            ->select('pg.only_specific_dates', 'pd.date', 'pd.start_time', 'pd.end_time')
            ->join('ext_productGroup AS pg', 'pg.id', '=', 'pd.product_group_id')
            ->where('pg.ext_bookableProduct_id', '=', $bookableProduct['id'])
            ->whereNull('pd.deleted_at')
            ->whereNull('pg.deleted_at')
            ->get();
        $endabledDates = [];
        $disabledDates = [];
        foreach($dates as $d){
            if($d->only_specific_dates) {
                $endabledDates[] = $d->date;
            }
            else {
                $disabledDates[] = $d->date;
            }
        }

        // Pull the data all together into one object
        $data = [
            'page' => Page::where('id', 1)->first()->toArray(),
            'product' => $bookableProduct,
            'calendar' => [
                'dateRange' => true,//count($endabledDates) == 0,
                'enabledDates' => $endabledDates,
                'disabledDates' => $disabledDates,
                // There is also option to highlight dates but we are not using that now
                'highlighted' => [],
                'date_translations' => $this->utilService->getDatePickerTranslations()
            ]
        ];

        return $data;
    }

    public function getProductsForDates($bookableProductId = 0, $start = '', $end = '') {
        $result = [];
        if($start != '' && $end != '') {
          $time = $this->getTimeInfo($start, $end);
    
          $productGroups = ProductGroup::where('ext_bookableProduct_id', $bookableProductId)->whereNull('deleted_at')->get()->toArray();
    
          foreach($productGroups as $productGroup) {
            // If flag is set for specific dates then we need to see if multiple events are on same date.
            // if($productGroup['only_specific_dates'] == 1) {
            //   $res = $this->getProductGroupForEventsWithPaymentInfo($productGroup, null, $start, $end, $time['timeInfo'], $time['nights']);
            //   $result = array_merge($result, $res);
            // }
            // else {
              $res = $this->getProductGroupWithPaymentInfo($productGroup, null, $start, $end, $time['timeInfo'], $time['nights']);
              if(isset($res)) {
                $result[] = $res;
              }
            // }
            
          }
        }
        return $result;
    }

    public function getBookingItems(string $payment_provider_id) : array {
        $result = DB::select("SELECT 
                ebi.*,
                p.totalQuantity, 
                CAST(NVL(i.bookedQuantity,0) AS SIGNED) AS bookedQuantity, 
                CAST(NVL(i2.tempQuantity,0) AS SIGNED) AS tempQuantity,
                p.totalQuantity - CAST(NVL(i.bookedQuantity,0) AS SIGNED) - CAST(NVL(i2.tempQuantity,0) AS SIGNED) AS bookableCount
            FROM ext_tempBookingItem ebi
            JOIN product p on p.id = ebi.product_id
            -- BookingItem
            LEFT JOIN (
                SELECT SUM(bi.quantity) AS bookedQuantity, bi.productId, bi.startTime, bi.endTime
                    FROM bookingitem bi
                    WHERE bi.bookingId NOT IN (SELECT id FROM booking WHERE STATUS = 'Makulerad')
                GROUP BY bi.productId, bi.startTime, bi.endTime) i ON 
                    i.productId = p.id
                    AND (
                        i.startTime BETWEEN ebi.startTime AND ebi.endTime
                        OR i.endTime BETWEEN ebi.startTime AND ebi.endTime
                        OR (i.startTime <= ebi.startTime AND i.endTime >= ebi.endTime)
                        OR (i.startTime >= ebi.startTime AND i.endTime <= ebi.endTime))
            -- TempBookingItem
            LEFT JOIN (SELECT SUM(tbi.quantity) AS tempQuantity, tbi.product_id, tbi.startTime, tbi.endTime, tbi.id
                FROM ext_tempBookingItem tbi
                WHERE 
                    (tbi.waiting_for_payment_confirmation IS NOT NULL OR tbi.updated_at > DATE_SUB(NOW(),INTERVAL 15 MINUTE))
                    AND 
                    tbi.deleted_at IS NULL
                GROUP BY tbi.product_id, tbi.startTime, tbi.endTime, tbi.id) i2 ON 
                    i2.product_id = p.id 
                    AND (
                        i2.startTime BETWEEN ebi.startTime AND ebi.endTime
                        OR i2.endTime BETWEEN ebi.startTime AND ebi.endTime
                        OR (i2.startTime <= ebi.startTime AND i2.endTime >= ebi.endTime)
                        OR (i2.startTime >= ebi.startTime AND i2.endTime <= ebi.endTime))
                    AND i2.id <> ebi.id
            WHERE ebi.payment_provider_id = '$payment_provider_id'
            AND ebi.deleted_at IS NULL");
        return $result;
    }

    public function getProductToBook($item, $start = '', $end = '', $uid = '') {
        $leftToBook = $item['count'];
        $time = $this->getTimeInfo($start, $end);
        $productGroup = ProductGroup::where('id', $item['productGroupId'])->whereNull('deleted_at')->first()->toArray();
        $products = $this->getProductsForGroup($productGroup['id'], $start, $end);
        $bookingItems = [];
        foreach($products as $product) {
            $toBook = $product['bookableCount'] >= $leftToBook ? $leftToBook : $product['bookableCount'];
            if($toBook > 0 && $leftToBook > 0) {
                $bookingItems[] = $this->createBookingItem($product, $item, $time, $toBook, $start, $end, $uid, $productGroup['title']);
                $leftToBook -= $toBook;
            }
        }
        // If I get here and leftToBook is not 0 then there are not enough products to book.
        if($leftToBook > 0) {
            // We do not want to book anything
            $bookingItems = [];
            // Need to show error to user.
        }
        return $bookingItems;
    }

    /**
     * Get how much can be booked at product.
     * @param $productId int the id for product
     * @param $start string 'yyyy-mm-dd' date, start of booking
     * @param $end string 'yyyy-mm-dd' date, end of booking
     * @param $uid int the uid of the booking
     */
    public function getProductBookableCount(int $productId, string $start, string $end, string $uid) : int {
        $productId = intval($productId);
        $result = 0;
        // Only look at temp items newer than 15 min
        $newer = Carbon::now()->subMinute(15)->format('Y-m-d H:i:s');

        $sqlResult = DB::select("SELECT 
                        p.id, 
                        p.totalQuantity, 
                        CAST(NVL(i.bookedQuantity,0) AS SIGNED) AS bookedQuantity, 
                        CAST(NVL(i2.tempQuantity,0) AS SIGNED) AS tempQuantity 
                      FROM product p
                      -- BookingItem
                      LEFT JOIN (SELECT SUM(bi.quantity) AS bookedQuantity, bi.productId FROM bookingitem bi
                          WHERE (
                            bi.startTime BETWEEN '$start' AND '$end'
                            OR bi.endTime BETWEEN '$start' AND '$end'
                            OR (bi.startTime <= '$start' AND bi.endTime >= '$end')
                            OR (bi.startTime >= '$start' AND bi.endTime <= '$end'))
                            AND bi.bookingId NOT IN (SELECT id FROM booking WHERE STATUS = 'Makulerad')
                          GROUP BY bi.productId) i ON i.productId = p.id
                      -- TempBookingItem
                      LEFT JOIN (SELECT SUM(tbi.quantity) AS tempQuantity, tbi.product_id FROM ext_tempBookingItem tbi
                          WHERE (
                            tbi.startTime BETWEEN '$start' AND '$end'
                            OR tbi.endTime BETWEEN '$start' AND '$end'
                            OR (tbi.startTime <= '$start' AND tbi.endTime >= '$end')
                            OR (tbi.startTime >= '$start' AND tbi.endTime <= '$end'))
                            AND (tbi.waiting_for_payment_confirmation IS NOT NULL OR tbi.updated_at > '$newer')
                            AND tbi.deleted_at IS NULL
                            AND tbi.uid <> '$uid'
                          GROUP BY tbi.product_id) i2 ON i2.product_id = p.id
                      WHERE p.id = $productId
                      AND p.hidden_product = 0
                      ORDER BY disPos");

        // We want to loop over the data coming from the database and get the count. We only expect 1 row max to be returned but DB::select returns array.
        foreach($sqlResult as $product) {
            $result += $product->totalQuantity - $product->bookedQuantity - $product->tempQuantity;
        }
        return $result;
    }

    private function createBookingItem($p, $item, $time, $count, $start, $end, $uid, $productGroupTitle) {
        $product = Product::where('id', $p['id'])->first()->toArray();
        $payment = $this->getPaymentForForProduct($p['id'], $item['optionId']);
        
        if(isset($item['extraId'])) {
            $product = ['id' => $payment['id']];
            $count = $item['count'];
        }

        $title = '';
        if(isset($payment['unit']) && $payment['unit'] == 'Dygn') {
          $n = $time['nights'] == 0 ? 1 : $time['nights'];
          $sum = $count * $payment['price'] * $n;
          $title = $productGroupTitle . ': ' . $item['timeInfo'];
        }
        else {
          $n = 1;
          $sum = $count * $payment['price'];
          $title = $payment['title'] . ' - ' .$productGroupTitle . ' ' . $item['timeInfo'];
        }

        $bookingItem = [
            'product_id' => isset($product['id']) ? $product['id'] : 0,
            'productGroup_id' => $item['productGroupId'],
            'quantity' => $count,
            'days' => $n,
            'sum' => $sum,
            'tax' => isset($payment['tax']) ? $payment['tax'] : 0,
            'price' => isset($payment['price']) ? $payment['price'] : 0,
            'startTime' => isset($start) ? $start : 0,
            'endTime' => isset($end) ? $end : 0,
            'externComments' => $title,
            'project' => $item['timeInfo'], 
            'uid' => $uid,
            'payment_provider_id' => '',
            'created_at' => Carbon::now(),
            'updated_at' => Carbon::now(),
            'eventUid' => $item['eventUid'],
            'ext_pg_paymentItem_id' => $payment['itemId']
        ];
        return $bookingItem;
    }

    private function getTimeInfo($start, $end) {
        $s_date = new DateTime($start);
        $e_date = new DateTime($end);

        //$dd = $

        $interval = $s_date->diff($e_date);
        $nights = $interval->days;
        if($interval->h > 12) {
          $nights++;
        }
        $nightsText = $nights == 1 ? 'natt' : 'nätter';
        if($nights == 0) {
            $timeInfo = $this->utilService->getDateString($s_date);
        }
        else {
            $timeInfo = $this->utilService->getDateString($s_date) . ' - ' . $this->utilService->getDateString($e_date) . ': ' . $nights . ' ' . $nightsText;
        }
        return ['nights' => $nights, 'timeInfo' => $timeInfo];
    }

    public function getProductGroupForEventsWithPaymentInfo($productGroup = [], $productGroupId = null, $start, $end, $timeInfo, $nights) {
      // Since we get here if it is by date only we need to check and see how many events are on current date and create entry for all of them
      $productDates = ProductDate::where('product_group_id', $productGroup['id'])->where('date', $start)->whereNull('deleted_at')->get()->toArray();
      $result = [];
      foreach($productDates as $productDate) {
        $item = $this->getProductGroupWithPaymentInfo($productGroup, $productGroupId, $start, $end, $timeInfo, $nights);
        $item['timeInfo'] .= ' ' . substr($productDate['start_time'],0,5) . ' - ' . substr($productDate['end_time'],0,5);
        $item['eventUid'] = $productDate['event_uid'];
        $result[] = $item;
      }
      return $result;
    }

    public function getProductGroupWithPaymentInfo($productGroup = [], $productGroupId = null, $start, $end, $timeInfo, $nights) {
        if(isset($productGroupId)) {
          $productGroup = ProductGroup::where('id', '=', $productGroupId)->whereNull('deleted_at')->first()->toArray();
        }

        $products = $this->getProductsForGroup($productGroup['id'], $start, $end);
    
        $product = $this->getSingleFreeProductFromGroup($products);
        $item = null;
        if(isset($product) && count($product) > 0) {
          $payment = $this->getDefaultPaymentForForProduct($product['id']);
          $paymentObject = PaymentObject::where('product_group_id', $productGroup['id'])->whereNull('deleted_at')->first();
          $paymentItems = $paymentObject->items()->get();
    
          $payment_options = $paymentObject->toArray();
          $payment_options['options'] = $paymentItems->toArray();
    
    
          $unit = 'sek';
          if(isset($payment['unit']) && $payment['unit'] == 'Dygn') {
            $unit = 'sek/natt';
          }
          else if(isset($payment['unit']) && $payment['unit'] == 'Styck') {
            $unit = 'sek';
          }
          
          $payment_options['full_text'] = __('fullybooked');
          $payment_options['available_count'] = $this->getTotalProductCountForGroup($products);
    
          foreach($payment_options['options'] as &$option) {
            if($option['title'] == '') {
              $option['title'] = $option['price']." ".$unit;
            }
            else {
              $option['title'] .= " (".$option['price']." ".$unit.")";
            }
            $option['min'] = 0;
            $option['value'] = 0;
            $option['max'] = $payment_options['available_count'];
          }
    
          $item = $productGroup;
          $item['price'] = 0;
          $item['timeInfo'] = $timeInfo;
          $item['nights'] = $nights;
          $item['payment_options'] = $payment_options;
        }
    
        return $item;
      }

      private function getTotalProductCountForGroup($products) {
          $total = 0;
          foreach($products as $product) {
            $total += $product['bookableCount'];
          }
          return $total;
      }

      private function getSingleFreeProductFromGroup($products, $count = 1) {
        // Start by getting list of all free products in group based on count
        $products = $this->getFreeProductsIdsFromGroup($products, $count);
        $productId = 0;
        $product = null;
        // We select the first product from the list.
        if(count($products) > 0) {
            $productId = $products[0]['id'];
        }
        if(isset($productId) && $productId > 0) {
            $product = Product::where('id', $productId)->first()->toArray();
        }
        return $product;
      }

      private function getFreeProductsIdsFromGroup($products, $count = 1, $alreadyUsedProducts = []) {
        $freeProducts = [];

        foreach($products as $product) {
            if(!in_array($product['id'], $alreadyUsedProducts)) {
                if($product['bookableCount'] >= $count) {
                    $freeProducts[] = json_decode(json_encode($product), true);
                }
            }
        }
        return $freeProducts;
      }

      /**
       * Pulling up default pricelist information for a product
       * @param $productId int, id of the product
       * @return array containing information about price, taxt, unit and name of product
       */
      public function getDefaultPaymentForForProduct(int $productId) : array {
        $payment = $this->itemToArray(DB::table('product AS p')
          ->select('p.id','pli.price','pli.tax','p.unit', 'p.productName')
          ->leftjoin('priceListItem AS pli', 'p.id', '=', 'pli.productId')
          ->leftjoin('priceList AS pl', 'pli.priceListId', '=', 'pl.id')
          ->where('p.id', '=', $productId)
          ->where('pl.defaultPriceList', '=', 1)
          ->first());
        return $payment;
      }

      /**
       * Pulling pricing information for a product by option
       * @param $productId int, id of the product
       * @param $optionId int, id of the option
       * @return array containing information about title, unit, productName, price, tax
       */
      public function getPaymentForForProduct($productId, $optionId) {
        $payment = $this->itemToArray(DB::table('product AS p')
          ->select('p.id', 'pi.id AS itemId', 'pi.title','p.unit', 'p.productName', 'pi.price', 'pi.tax')
          ->join('ext_pg_product AS pgp', 'pgp.product_id', '=', 'p.id')
          ->join('ext_pg_paymentObject AS po', 'po.product_group_id', '=', 'pgp.ext_productGroup_id')
          ->join('ext_pg_paymentItem AS pi', 'pi.payment_object_id', '=', 'po.id')
          ->where('p.id', '=', $productId)
          ->where('pi.id', '=', $optionId)
          ->whereNull('pi.deleted_at')
          ->first());
        return $payment;
      }

      /**
       * Function that gets all products for a product group and lists up how many can be booked for the product and how much is booked.
       * @param $productGroupId int the id of the product group we are looking at
       * @param $start string 'Y-m-d H:i:s' string with start date
       * @param $end string 'Y-m-d H:i:s' string with end date
       * @return array of products with max allowed and used counts.
       */
      private function getProductsForGroup($productGroupId, $start, $end) : array {
        $sqlResult = DB::select("SELECT 
                        p.id, 
                        p.totalQuantity, 
                        CAST(NVL(i.bookedQuantity,0) AS SIGNED) AS bookedQuantity, 
                        CAST(NVL(i2.tempQuantity,0) AS SIGNED) AS tempQuantity,
                        p.totalQuantity - CAST(NVL(i.bookedQuantity,0) AS SIGNED) - CAST(NVL(i2.tempQuantity,0) AS SIGNED) AS bookableCount
                      FROM product p
                      JOIN ext_pg_product pgp ON pgp.product_id = p.id
                      JOIN ext_productGroup pg ON pg.id = pgp.ext_productGroup_id
                      -- BookingItem
                      LEFT JOIN (SELECT SUM(bi.quantity) AS bookedQuantity, bi.productId FROM bookingitem bi
                          WHERE (
                            bi.startTime BETWEEN '$start' AND '$end'
                            OR bi.endTime BETWEEN '$start' AND '$end'
                            OR (bi.startTime <= '$start' AND bi.endTime >= '$end')
                            OR (bi.startTime >= '$start' AND bi.endTime <= '$end'))
                            AND bi.bookingId NOT IN (SELECT id FROM booking WHERE STATUS = 'Makulerad')
                          GROUP BY bi.productId) i ON i.productId = p.id
                      -- TempBookingItem
                      LEFT JOIN (SELECT SUM(tbi.quantity) AS tempQuantity, tbi.product_id FROM ext_tempBookingItem tbi
                          WHERE (
                            tbi.startTime BETWEEN '$start' AND '$end'
                            OR tbi.endTime BETWEEN '$start' AND '$end'
                            OR (tbi.startTime <= '$start' AND tbi.endTime >= '$end')
                            OR (tbi.startTime >= '$start' AND tbi.endTime <= '$end'))
                            AND (tbi.waiting_for_payment_confirmation IS NOT NULL OR tbi.updated_at > DATE_SUB(NOW(),INTERVAL 15 MINUTE))
                            AND tbi.deleted_at IS NULL
                          GROUP BY tbi.product_id) i2 ON i2.product_id = p.id
                      WHERE pg.id = $productGroupId
                      AND p.hidden_product = 0
                      ORDER BY disPos");

        $result = [];
        foreach($sqlResult as $item) {
            $result[] = json_decode(json_encode($item), true);
        }

        return $result;
      }

    private function itemToArray($item) {
        if($item == null) {
            $item = [];
        }
        else if(isset($item) && !is_object($item)) {
            $item = $item->toArray();
        }
        else {
            $item = json_decode(json_encode($item), true);
        }
        return $item;
    }
}